@@ -56,20 +56,39 @@ def split_sub_matrix(mat: torch.Tensor, *sizes):
     return mat
 
 
-def cartesian_product(left: torch.Tensor, right: torch.Tensor):
+def cartesian_product(left: torch.Tensor, right: torch.Tensor=None):
     """
     笛卡尔积
     example:
         cartesian_product(torch.arange(0,3),torch.arange(0,5))
         >> (tensor([0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2]),
             tensor([0, 1, 2, 3, 4, 0, 1, 2, 3, 4, 0, 1, 2, 3, 4]))
-    :param mat:
-    :return: 两个矩阵
+    :param left:
+    :param right: if None, wile be left X left
+    :return: two tensors
     """
     # za = mat.repeat_interleave(mat.shape[0], dim=0)
     # zb = mat.repeat([mat.shape[0], 1])
+    if right is None:
+        right = left
+
     nleft = left.repeat_interleave(right.shape[0], dim=0)
     nright = right.repeat(*[item if i == 0 else 1 for i, item in enumerate(left.shape)])
     return nleft, nright
 
-# def similarity(ipt1,ipt2=None,norm)
\ No newline at end of file
+# def similarity(ipt1,ipt2=None,norm)
+
+def cat_then_split(op,tensors:list):
+    """
+    ta,tb,tc = cat_then_split(lambda x:x.to(device),[ta,tb,tc])
+    alogits,blogits,clogits = cat_then_split(model,[ta,tb,tc])
+
+    :param op:
+    :param tensors:
+    :return:
+    """
+
+
+    res = op(torch.cat(tensors)) # type: torch.Tensor
+    return res.split_with_sizes([i.shape[0] for i in tensors])
+
